<?php

// IPplan v4.04
// Aug 24, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// test if ranges overlap
function TestDuplicateRange($ds, $rangeaddr, $rangesize, $cust) {

    $result=&$ds->ds->Execute("SELECT rangeaddr
                               FROM range
                               WHERE (($rangeaddr BETWEEN rangeaddr AND 
                                  rangeaddr + rangesize - 1) OR
                                  ($rangeaddr < rangeaddr AND 
                                  $rangeaddr+$rangesize > 
                                      rangeaddr + rangesize - 1)) AND
                                  customer=$cust");

    if ($result->FetchRow()) {
       return 1;
    }
}

require_once("../ipplanlib.php");
require_once("../adodb/adodb.inc.php");
require_once("../class.dbflib.php");
require_once("../layout/class.layout");
require_once("../auth.php");

$auth = new SQLAuthenticator(REALM, REALMERROR);

// And now perform the authentication
$grps=$auth->authenticate();

// set language
myLanguage($HTTP_COOKIE_VARS['ipplanLanguage']);

setdefault("window",array("bgcolor"=>"white"));
//setdefault("table",array("cellpadding"=>"0"));
//setdefault("text",array("size"=>"2"));

$title=my_("Create a new range or supernet/summary");
newhtml($w);

insert($w, wheader($title));
myheading($w, $title, "../index.php", true);

// explicitly cast variables as security measure against SQL injection
$cust=floor($cust);
$areaindex=floor($areaindex);
$size=floor($size);
$ds=new IPplanDbf() or myError($w, my_("Could not connect to database"));

if ($HTTP_POST_VARS) {
   // save the last customer used
   // must set path else Netscape gets confused!
   setcookie("ipplanCustomer","$cust",time() + 10000000, "/");

   $descrip=trim($descrip);

   if (strlen($descrip) == 0) {
      $formerror .= my_("You need to enter a description for the range")."<br>";
   }

   $base=inet_aton($ipaddr);

   if (!$ipaddr)
      $formerror .= my_("Range address may not be blank")."<br>";
   else if (testIP($ipaddr))
      $formerror .= my_("Invalid range address")."<br>";
   else if (!$size)
      $formerror .= my_("Size may not be zero")."<br>";
   else if (TestDuplicateRange($ds, $base, $size, $cust))
      $formerror .= my_("Range cannot be created - overlaps with existing range")."<br>";

   if ($size > 1) {
      if (TestBaseAddr(inet_aton3($ipaddr), $size)) {
         $formerror .= my_("Invalid base address")."<br>";
      }
   }

   if (!$formerror) {
      // check if user belongs to customer admin group
      $result=$ds->GetCustomerGrp($cust);
      // can only be one row - does not matter if nothing is 
      // found as array search will return false
      $row = $result->FetchRow();
      if (!in_array($row["admingrp"], $grps)) {
         myError($w, my_("You may not create a range for this customer as you are not a member of the customers admin group"));
      } 

      // cast type correctly to prevent blank area!
      $areaindex=(int)$areaindex;

      $ds->DbfTransactionStart();
      // the fact that the range is unique prevents the range
      // being added to more than one area!
      $result=&$ds->ds->Execute("INSERT INTO range
                                (rangeaddr, rangesize, areaindex, descrip,
                                 customer)
                             VALUES
                                ($base, $size, $areaindex,
                                ".$ds->ds->qstr($descrip).",
                                 $cust)") and
      $ds->AuditLog(sprintf(my_("User %s created new range %s size %u in areaindex %u cust %u"),
               $HTTP_SERVER_VARS[AUTH_VAR], $ipaddr, $size, $areaindex, $cust));
 
      if ($result) {
         $ds->DbfTransactionEnd();
         insert($w,textbr(my_("Range created")));
         $ipaddr=""; $descrip="";
      }
      else {
         $ds->DbfTransactionRollback();
         $formerror .= my_("Range could not be created - probably duplicate name")."<br>";
      }
   }
}

insert($w,text($formerror, array("color"=>"red")));
 
// display opening text
insert($w,textbrbr(my_("Create a new range or supernet/summary by entering the base (network) address. A range is an aggregation of a number of subnets which can ultimately be used to reduce the number of routing entries in routing tables. Ranges are essentially the same as a normal subnet, just bigger with a wider view.")));

// start form
insert($w, $f1 = form(array("name"=>"THISFORM",
                            "method"=>"get",
                            "action"=>"$PHP_SELF")));

// ugly kludge with global variable!
$displayall=TRUE;
$cust=myCustomerDropDown($ds, $f1, $cust, $grps) or myError($w, my_("No customers"));
$areaindex=myAreaDropDown($ds, $f1, $cust, $areaindex);

insert($w, $f2 = form(array("name"=>"ENTRY",
                            "method"=>"post",
                            "action"=>$PHP_SELF)));

// save customer name for actual post of data
insert($f2,hidden(array("name"=>"cust",
                        "value"=>"$cust")));
// save area index for actual post of data
insert($f2,hidden(array("name"=>"areaindex",
                        "value"=>"$areaindex")));

insert($f2,textbrbr(my_("Range address")));

insert($f2,input_text(array("name"=>"ipaddr",
                           "value"=>"$ipaddr",
                           "size"=>"15",
                           "maxlength"=>"15")));

insert($f2,textbrbr(my_("Description")));
insert($f2,input_text(array("name"=>"descrip",
                            "value"=>"$descrip",
                            "size"=>"80",
                            "maxlength"=>"80")));
insert($f2,textbrbr(my_("Range size/mask")));

insert($f2,selectbox(array("4"=>"255.255.255.252/30 - 4 hosts",
                           "8"=>"255.255.255.248/29 - 8 hosts",
                           "16"=>"255.255.255.240/28 - 16 hosts",
                           "32"=>"255.255.255.224/27 - 32 hosts",
                           "64"=>"255.255.255.192/26 - 64 hosts",
                           "128"=>"255.255.255.128/25 - 128 hosts",
                           "256"=>"255.255.255.0/24 - 256 hosts (class C)",
                           "512"=>"255.255.254.0/23 - 512 hosts",
                           "1024"=>"255.255.252.0/22 - 1k hosts",
                           "2048"=>"255.255.248.0/21 - 2k hosts",
                           "4096"=>"255.255.240.0/20 - 4k hosts",
                           "8192"=>"255.255.224.0/19 - 8k hosts",
                           "16384"=>"255.255.192.0/18 - 16k hosts",
                           "32768"=>"255.255.128.0/17 - 32k hosts",
                           "65536"=>"255.255.0.0/16 - 64k hosts (class B)",
                           "131072"=>"255.254.0.0/15 - 128k hosts",
                           "262144"=>"255.252.0.0/14 - 256k hosts"),
                     array("name"=>"size"),
                     $size));

insert($f2,generic("br"));
insert($f2,submit(array("value"=>my_("Submit"))));
insert($f2,freset(array("value"=>my_("Clear"))));

printhtml($w);

?>
