<?php

// IPplan v4.04
// Aug 24, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

require_once("../ipplanlib.php");
require_once("../adodb/adodb.inc.php");
require_once("../class.dbflib.php");
require_once("../layout/class.layout");
require_once("../auth.php");

$auth = new SQLAuthenticator(REALM, REALMERROR);

// And now perform the authentication
$grps=$auth->authenticate();

// save the last customer used
// must set path else Netscape gets confused!
setcookie("ipplanCustomer","$cust",time() + 10000000, "/");
setcookie("ipplanGroup","$admingrp",time() + 10000000, "/");

// set language
myLanguage($HTTP_COOKIE_VARS['ipplanLanguage']);

setdefault("window",array("bgcolor"=>"white"));
//setdefault("table",array("cellpadding"=>"0"));
//setdefault("text",array("size"=>"2"));

$title=my_("Create subnet results");
newhtml($w);

insert($w, wheader($title));
myheading($w, $title, "../index.php", true);

// explicitly cast variables as security measure against SQL injection
$cust=floor($cust);
$size=floor($size);
$num=(int)$num;
$descrip=trim($descrip);

// must only check once - might need to create multiple nets
if (strlen($descrip) == 0) {
   $nodescrip=1;
}
if ($num <1 or $num > 255) {
   myError($w, my_("Number of subnets to create is out of bounds."));
}

// basic sequence is connect, search, interpret search
// result, close connection

$ds=new IPplanDbf() or myError($w, my_("Could not connect to database"));

// error checks
if (!$ipaddr)
   myError($w, my_("IP address may not be blank"));
else if (testIP($ipaddr))
   myError($w, my_("Invalid IP address"));
else if (!$size)
   myError($w, my_("Size may not be zero"));
elseif ($size > 1) {
   if (TestBaseAddr(inet_aton3($ipaddr), $size)) {
      myError($w, my_("Invalid base address!"));
   }
}

if ($addhostinfo) {
   // increase time limit for scans - will have no effect in safe mode is on
   @set_time_limit(90);
}

for ($i=1; $i <= $num; $i++) {
   // work out new base for each iteration of loop
   $base=inet_aton($ipaddr)+(($i-1)*$size);
   // create description if none
   if ($nodescrip) {
      $descrip="NET-".str_replace(".", "-", inet_ntoa($base));
   }

   // test if subnet to create is within bounds
   foreach ($grps as $value) {
      if ($extst = $ds->TestBounds($base, $size, $value)) {
         // got an overlap, allowed to create
         break;
      }
   }
   // could not find new subnet within any of the defined bounds
   // so do not create
   if (!$extst) {
      insert($w,text(sprintf(my_("Subnet %s not created - out of defined authority boundary"), inet_ntoa($base))."<br>", array("color"=>"red")));
      continue;
   }
 
   $restemp=$ds->GetDuplicateSubnet($base, $size, $cust);
   if ($restemp->FetchRow()) {
      insert($w,text(sprintf(my_("Subnet %s could not be created - possibly overlaps with an existing subnet"), inet_ntoa($base))));
      insert($w,textbr());
   }
   else {

      // find subnets from other customers that overlap
      $result=$ds->GetDuplicateSubnetAll($base, $size, $grps);
      if ($row=$result->FetchRow()) {
         insert($w,textb(my_("WARNING: ")));
         insert($w,text(sprintf(my_("Subnet %s overlaps with the following subnets from other customers.  This may not be a problem as the subnet may occur in both customers routing tables:"),inet_ntoa($base))));
         insert($w,block("<p>"));

         // create a table
         insert($w,$tbig = table(array("cols"=>"1", "bgcolor"=>"#00CCFF")));
         insert($tbig,$c = cell());
         insert($c,$t = table(array("cols"=>"3",
                                    "cellspacing"=>"1",
                                    "bgcolor"=>"#99CCFF")));
         // draw heading
         insert($t,$c = cell());
         insert($c,textb(my_("Base address")));
         insert($t,$c = cell());
         insert($c,textb(my_("Subnet description")));
         insert($t,$c = cell());
         insert($c,textb(my_("Customer")));


         do {
         setdefault("cell",array("bgcolor"=>color_flip_flop()));

            insert($t,$c = cell());
            insert($c,text(inet_ntoa($row["baseaddr"])));
 
            insert($t,$c = cell());
            insert($c,text($row["descrip"]));

            insert($t,$c = cell());
            insert($c,text($row["custdescrip"]));
         } while($row = $result->FetchRow());

         setdefault("cell", "");
         insert($w,block("<p>"));
      }

      // check if user belongs to customer admin group
      $result=$ds->GetCustomerGrp($cust);
      // can only be one row - does not matter if nothing is 
      // found as array search will return false
      $row = $result->FetchRow();
      if (!in_array($row["admingrp"], $grps)) {
         myError($w, my_("You may not create a subnet for this customer as you are not a member of the customers admin group"));
      }
 
      $ds->DbfTransactionStart();
      // use the first group user belongs to create subnet
      if ($id = $ds->CreateSubnet($base, $size, $descrip, $cust, $admingrp)) {
         $ds->AuditLog(sprintf(my_("User %s created new subnet %s size %u cust %u"),
                  $HTTP_SERVER_VARS[AUTH_VAR], inet_ntoa($base), 
                  $size, $cust));
         insert($w,text(sprintf(my_("Subnet %s created"), inet_ntoa($base))));
         // add link if script was called from findfree
         if ($findfree) {
             header("Location: ".base64_decode($findfree));
             exit;
         }
         insert($w,textbr());
      }
      else {
         myError($w, sprintf(my_("Subnet %s could not be created"), inet_ntoa($base)));
      }

      // fill new subnet with nmap
      if (NMAP != "" and $addnmapinfo and $size >= 4 and $size <= 1024) {
         if (ProcessNmap($ds, $base, $id, $size)) {
            insert($w,textbr());
            insert($w,text(my_("NMAP data not in XML format or XML not available")));
         }
      }
      // fill new subnet with DNS info
      else if ($addhostinfo and $size > 0) {
         // remember to skip broadcast and network addresses!
         for ($i=1; $i < $size-1; $i++) {
             $hname=gethostbyaddr(inet_ntoa($base+$i));
             if ($hname != inet_ntoa($base+$i))
                $ds->AddIP($base+$i, $id, "", "", "", "", $hname, "");
         }
      }
      $ds->DbfTransactionEnd();
   }
}

printhtml($w);

function ProcessNmap($ds, $base, $id, $size) {

   global $addhostinfo;

   $resarr=array();
   if ($addhostinfo)
      $command=NMAP." -sP ".inet_ntoa($base)."/".inet_bits($size)." -oX -";
   else
      $command=NMAP." -n -sP ".inet_ntoa($base)."/".inet_bits($size)." -oX -";

   exec($command, $resarr, $retval);
   if (!$retval) { // no error
      require_once("../xmllib.php");

      $input = implode("",$resarr);
      $xml_parser = new xml("HOST");
      if (!$xml_parser->parser) {
         return 1;  // XML parser failure - probably not compiled in
      }
      $output=$xml_parser->parse($input);
      if (!$output) {
         return 1;  // not XML format
      }

      foreach ($output as $value) {
         if ($value["STATUS"]["STATE"]=="up") {
            $newbase=inet_aton($value["ADDRESS"]["ADDR"]);
            if(empty($value["OSMATCH"]["NAME"])) {
               $newuser="";
            }
            else {
               $newuser=$value["OSMATCH"]["NAME"];
            }
            $newdescrip="active";
            if(empty($value["HOSTNAME"]["NAME"])) {
               $newhname="";
            }
            else {
               $newhname=$value["HOSTNAME"]["NAME"];
            }

            // check within range of subnet before adding to ignore
            // broadcast and network addresses
            if ($newbase > $base and $newbase < $base+$size-1) {
               $ds->AddIP($newbase, $id, $newuser, "", "", $newdescrip, $newhname, "");
            }
         }
      }
   }

   return 0;
}
?>
