<?php

// IPplan v4.04
// Aug 24, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

require_once("../ipplanlib.php");
require_once("../adodb/adodb.inc.php");
require_once("../class.dbflib.php");
require_once("../layout/class.layout");
require_once("../auth.php");

$auth = new SQLAuthenticator(REALM, REALMERROR);

// And now perform the authentication
$grps=$auth->authenticate();

// save the last customer used
// must set path else Netscape gets confused!
setcookie("ipplanCustomer","$cust",time() + 10000000, "/");

// set language
myLanguage($HTTP_COOKIE_VARS['ipplanLanguage']);

setdefault("window",array("bgcolor"=>"white"));
//setdefault("table",array("cellpadding"=>"0"));
//setdefault("text",array("size"=>"2"));

$title=my_("Results of your search for subnet to modify");
newhtml($w);

insert($w, wheader($title));
myheading($w, $title, "../index.php", true);

// explicitly cast variables as security measure against SQL injection
$cust=floor($cust);
$baseindex=floor($baseindex);
$areaindex=floor($areaindex);
$rangeindex=floor($rangeindex);
$block=(int)$block;
if (!$HTTP_GET_VARS) {
   myError($w, my_("You cannot reload or bookmark this page!"));
}

// basic sequence is connect, search, interpret search
// result, close connection
$ds=new IPplanDbf() or myError($w, my_("Could not connect to database"));

if ($action=="delete") {
   // check if user belongs to customer admin group
   $result=$ds->GetCustomerGrp($cust);
   // can only be one row - does not matter if nothing is 
   // found as array search will return false
   $row = $result->FetchRow();
   if (!in_array($row["admingrp"], $grps)) {
      myError($w, my_("You may not delete a subnet for this customer as you are not a member of the customers admin group"));
   } 
 
   // get info from base table
   $result=$ds->GetBaseFromIndex($baseindex);

   $row = $result->FetchRow();
   $size=$row["subnetsize"];
   $base=$row["baseaddr"];
   $baseip=inet_ntoa($row["baseaddr"]);

   // test if subnet to delete is within bounds
   foreach ($grps as $value) {
      if ($extst = $ds->TestBounds($base, $size, $value)) {
         // got an overlap, allowed to delete
         break;
      } 
   }
   // could not find new subnet within any of the defined bounds
   // so do not delete
   if (!$extst) { 
      myError($w,sprintf(my_("Subnet %s not deleted - out of defined authority boundary"), $baseip)."<br>");
   }

   $ds->DbfTransactionStart();
   if ($ds->DeleteSubnet($baseindex)) {
      $ds->AuditLog(sprintf(my_("User %s deleted subnet %s size %u customer cust %u"),
               $HTTP_SERVER_VARS[AUTH_VAR], $baseip, $size, $cust));

      $ds->DbfTransactionEnd();
      insert($w,textbr(my_("Subnet deleted")));
   }
   else
      insert($w,textbr(my_("Subnet could not be deleted")));
}

// set start and end address according to range
if ($rangeindex) {
    // should only return one row here!
    $result=$ds->GetRange($cust, $rangeindex);
    $row = $result->FetchRow();
 
    $start=inet_ntoa($row["rangeaddr"]);
    $end=inet_ntoa($row["rangeaddr"]+$row["rangesize"]-1);
    $site=" (".$row["descrip"].")";
}
else {
    if ($ipaddr) {
       $start = completeIP($ipaddr, 1);
       $end = completeIP($ipaddr, 2);

       if (testIP($start) or testIP($end)) {
          myError($w, my_("Invalid IP address!"));
       }
    }
    else {
       $start=DEFAULTROUTE;
       $end=ALLNETS;
    }
}

$startnum=inet_aton($start);
$endnum=inet_aton($end);

$custdescrip=$ds->GetCustomerDescrip($cust);

if ($areaindex and !$rangeindex) {
   insert($w,heading(3, sprintf(my_("Search for IP subnets between multiple ranges for customer '%s'"), $custdescrip)));
   $result = $ds->GetBaseFromArea($areaindex, $descrip, $cust);
}
else {
   insert($w,heading(3, sprintf(my_("Search for IP subnets between %s and %s %s for customer '%s'"), $start, $end, $site, $custdescrip)));
   $result = $ds->GetBase($startnum, $endnum, $descrip, $cust);
}
if ($ipaddr) {
   insert($w,textb(my_("IP address filter: ")));
   insert($w,textbr($ipaddr));
}
if ($descrip) {
   insert($w,textb(my_("Description filter: ")));
   insert($w,textbr($descrip));
}

$totcnt=0;
$vars="";
// fastforward till first record if not first block of data
while ($block and $totcnt < $block*MAXTABLESIZE and
       $row = $result->FetchRow()) {
    $vars=DisplayBlock($w, $row, $totcnt, 
                        "&cust=".$cust."&areaindex=".$areaindex.
                        "&rangeindex=".$rangeindex."&ipaddr=".$ipaddr.
                        "&descrip=".$descrip);
    $totcnt++;
}
insert($w,block("<p>"));

// create a table
insert($w,$tbig = table(array("cols"=>"1", "bgcolor"=>"#00CCFF")));
insert($tbig,$c = cell());
insert($c,$t = table(array("cols"=>"6",
                           "cellspacing"=>"1",
                           "bgcolor"=>"#99CCFF")));
// draw heading
insert($t,$c = cell());
if (!empty($vars))
    insert($c,anchor($vars, "<<"));
insert($c,textb(my_("Base address")));
insert($t,$c = cell());
insert($c,textb(my_("Subnet size")));
insert($t,$c = cell());
insert($c,textb(my_("Subnet mask")));
insert($t,$c = cell());
insert($c,textb(my_("Description")));
insert($t,$c = cell());
insert($c,textb(my_("Admin group")));
insert($t,$ck = cell());
insert($ck,textb(my_("Action")));

 
$cnt=0;
while($row = $result->FetchRow()) {
setdefault("cell",array("bgcolor"=>color_flip_flop()));

    insert($t,$c = cell());
    insert($c,anchor("displaysubnet.php?baseindex=".$row["baseindex"]."&cust=".$cust, 
                     inet_ntoa($row["baseaddr"])));
    if ($row["subnetsize"] == 1) {
        insert($t,$c = cell());
        insert($c,text(my_("Host")));
    }
    else {
       insert($t,$c = cell());
       insert($c,text($row["subnetsize"]));
    }

    insert($t,$c = cell());
    insert($c,text(inet_ntoa(inet_aton(ALLNETS)+1 -
                        $row["subnetsize"])."/".inet_bits($row["subnetsize"])));

    insert($t,$c = cell());
    insert($c,text($row["descrip"]));

    insert($t,$c = cell());
    insert($c,block("<small>"));
    insert($c,anchor("../admin/displayaccessform.php?grp=".urlencode($row["admingrp"]), 
                     $row["admingrp"]));
    insert($c,block("</small>"));

    insert($t,$c = cell());
    insert($c,block("<small>"));
    insert($c,anchor($PHP_SELF."?baseindex=".$row["baseindex"]."&cust=".$cust.
                     "&areaindex=".$areaindex."&rangeindex=".$rangeindex.
                     "&descrip=".urlencode($descrip)."&block=".$block.
                     "&ipaddr=".$ipaddr."&action=delete", 
                     my_("Delete Subnet"),
                     $ipplanParanoid ? array("onclick"=>"return confirm('".my_("Are you sure?")."')") : FALSE));
    insert($c,block(" | "));
    insert($c,anchor("modifysubnet.php?baseindex=".$row["baseindex"].
                     "&cust=".$cust."&descrip=".urlencode($row["descrip"]).
                     "&grp=".urlencode($row["admingrp"]), 
                     my_("Modify/Copy/Move subnet details")));
    insert($c,block("</small>"));

   if ($totcnt % MAXTABLESIZE == MAXTABLESIZE-1)
      break;
   $cnt++;
   $totcnt++;
}
insert($w,block("<p>"));

if (!$cnt) {
   myError($w, my_("Search found no matching entries"));
}

$vars="";
$printed=0;
while ($row = $result->FetchRow()) {
    $totcnt++;
    $vars=DisplayBlock($w, $row, $totcnt, 
            "&cust=".$cust."&areaindex=".$areaindex.
            "&rangeindex=".$rangeindex."&ipaddr=".$ipaddr.
            "&descrip=".$descrip);
    if (!empty($vars) and !$printed) {
        insert($ck,anchor($vars, ">>"));
        $printed=1;
    }
}

$result->Close();
printhtml($w);

?>
