<?php

// IPplan v4.04
// Aug 24, 2001
//
// Modified by Tony D. Koehn February 2003
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

require_once("../ipplanlib.php");
require_once("../adodb/adodb.inc.php");
require_once("../class.dbflib.php");
require_once("../layout/class.layout");
require_once("../auth.php");

$auth = new SQLAuthenticator(REALM, REALMERROR);

// And now perform the authentication
$grps=$auth->authenticate();

// save the last customer used
// must set path else Netscape gets confused!
setcookie("ipplanCustomer","$cust",time() + 10000000, "/");

// set language
myLanguage($HTTP_COOKIE_VARS['ipplanLanguage']);

setdefault("window",array("bgcolor"=>"white"));
//setdefault("table",array("cellpadding"=>"0"));
//setdefault("text",array("size"=>"2"));

$title=my_("DNS Reverse Zones");
newhtml($w);

insert($w, wheader($title));
myheading($w, $title, "../index.php", true);

// explicitly cast variables as security measure against SQL injection
$cust=floor($cust);
$zoneid=floor($zoneid);
$size=floor($size);
$serialnum=(int)$serialnum;
$ttl=(int)$ttl;
$refresh=(int)$refresh;
$retry=(int)$retry;
$expire=(int)$expire;
$minimum=(int)$minimum;

//if (!$HTTP_GET_VARS) {
//   myError($w, my_("You cannot reload or bookmark this page!"));
//}

// basic sequence is connect, search, interpret search
// result, close connection
$ds=new IPplanDbf() or myError($w, my_("Could not connect to database"));

// CHECK Actions First

// ##################### Start OF DELETE ##############################
if ($action=="delete") {
    // check if user belongs to customer admin group
    $result=$ds->GetCustomerGrp($cust);
    // can only be one row - does not matter if nothing is 
    // found as array search will return false
    $row = $result->FetchRow();
    if (!in_array($row["admingrp"], $grps)) {
        myError($w, my_("You may not delete a zone as you are not a member of the customers admin group"));
    } 

    // Log the Transaction.
    $ds->DbfTransactionStart();
    $result = $ds->ds->Execute("DELETE FROM zones 
            WHERE customer=$cust AND id=$zoneid") and
              $ds->ds->Execute("DELETE FROM zonedns
            WHERE id=$zoneid") and
        $ds->AuditLog(sprintf(my_("User %s Deleted zone %s."),
                    $HTTP_SERVER_VARS[AUTH_VAR], $zoneip));

    $ds->DbfTransactionEnd();

    if ($result) {
        $ds->DbfTransactionEnd();
        insert($w,textbr(my_("Zone deleted")));
        $zone="";
    }
    else {
        $ds->DbfTransactionRollback();
        $formerror .= my_("Zone could not be deleted.")."<br>";
    }
}
// ##################### END OF DELETE ##############################

// ##################### Start OF checks ##############################
if ($action=="add" or $action=="edit") {
    // check if user belongs to customer admin group
    $result=$ds->GetCustomerGrp($cust);
    // can only be one row - does not matter if nothing is 
    // found as array search will return false
    $row = $result->FetchRow();
    if (!in_array($row["admingrp"], $grps)) {
        myError($w, my_("You may not add a zone as you are not a member of the customers admin group"));
    } 

    // Error Checks
    if (!$zone)
        myError($w, my_("IP address may not be blank"));
    else if (testIP($zone))
        myError($w, my_("Invalid IP address"));
    else if (!$size)
        myError($w, my_("Size may not be zero"));
    else if ($size > 1) {
        if (TestBaseAddr(inet_aton3($zone), $size)) {
            myError($w, my_("Invalid base address!"));
        }
    }

    $cnt=0;
    for ($i = 1; $i < 11; $i++) {
        if ($hname[$i] and
                !preg_match("/[^ \t@()<>,]+\\.[^ \t()<>,.]+$/", $hname[$i])) {
            myError($w, sprintf(my_("Invalid hostname %u"), $i)."<br>");
        }
        if ($hname[$i]) {
            $cnt++;
        }
    }
    if ($cnt < 2) {
        myError($w, my_("Invalid zone - you need at least two nameservers"));
    }
    if (!is_numeric($ttl) or !is_numeric($refresh) or !is_numeric($retry) or
        !is_numeric($expire) or !is_numeric($minimum)) {
        myError($w, my_("Invalid domain timeout values"));
    }

    if (!$serialdate) {
        myError($w, my_("Serial Date can not be blank.  Use YYMMDD."));
    }
}
 
// ##################### Start OF add ##############################
if ($action=="add") {

    $zoneip = inet_aton($zone);

    // code not optimal - could have had extra key on zones table, but
    // would require additional space for key used only on insert
    $restemp=&$ds->ds->Execute("SELECT zoneip 
            FROM zones 
            WHERE customer=$cust AND zoneip = $zoneip");

    if ($restemp->FetchRow()) {
        insert($w,text(sprintf(my_("Zone %s could not be created - possibly duplicate zone"), $zone)));
        insert($w,textbr());
    }
    else {
        // Add to DB here.
        // Log the Transaction.
        $ds->DbfTransactionStart();

        $result = &$ds->ds->Execute("INSERT into zones 
                (customer, zoneip, zone, zonesize, serialdate, 
                 serialnum, ttl, refresh, retry, expire, minimum, 
                 lastmod, zonefilepath1, zonefilepath2) ".
                "VALUES ($cust, $zoneip,".
                $ds->ds->qstr($zone).", $size,".
                $ds->ds->qstr($serialdate).", $serialnum,".
                $ttl.",".
                $refresh.",".
                $retry.",".
                $expire.",".
                $minimum.",".
                $ds->ds->DBTimeStamp(time()).",".
                $ds->ds->qstr($zonepath).",".
                $ds->ds->qstr($seczonepath).")" );

        // did not fail due to key error?
        if ($result) {
            if (DBF_TYPE=="mysql" or DBF_TYPE=="maxsql") {
                $zoneid=$ds->ds->Insert_ID();
            }
            else {
                // emulate getting the last insert_id
                $result=&$ds->ds->Execute("SELECT id 
                        FROM zones
                        WHERE customer=$cust AND 
                        zoneip=$zoneip");
                $temprow = $result->FetchRow();
                $zoneid=$temprow["id"];
            }

            // add reverse DNS into zonedns table
            for ($i = 1; $i < 11; $i++) {
                if ($hname[$i]) {
                    $hnametemp=$hname[$i];

                    // add DNS records
                    $result=&$ds->ds->Execute("INSERT INTO zonedns
                            (id, hname, horder)
                            VALUES
                            ($zoneid,
                             ".$ds->ds->qstr($hnametemp).",
                             $i)");
                }
            }
        }

        if ($result) {
            $ds->AuditLog(sprintf(my_("User %s added zone %s."),
                        $HTTP_SERVER_VARS[AUTH_VAR], $zoneip));

            $ds->DbfTransactionEnd();
            insert($w,textbr(my_("Zone created")));
            $zone="";
        }
        else {
            $ds->DbfTransactionRollback();
            $formerror .= my_("Zone could not be created - probably duplicate name")."<br>";
        }
    }
}
// ##################### END OF Add ##############################

// ##################### Start OF Edit ##############################
if ($action=="edit") {

    $zoneip = inet_aton($zone);
    // Updated DB here.
    // Log the Transaction.
    $ds->DbfTransactionStart();

    $result = &$ds->ds->Execute("UPDATE zones SET zoneip=$zoneip".
            ",zone=".$ds->ds->qstr($zone).",zonesize=$size".
            ",serialdate=".$ds->ds->qstr($serialdate).", serialnum=$serialnum".
            ",ttl=".$ttl.
            ",refresh=".$refresh.
            ",retry=".$retry.
            ",expire=".$expire.
            ",minimum=".$minimum.
            ",lastmod=".$ds->ds->DBTimeStamp(time()).
            ",zonefilepath1=".$ds->ds->qstr($zonepath).
            ",zonefilepath2=".$ds->ds->qstr($seczonepath).
            "WHERE customer=$cust AND id=".$zoneid );

    // delete all the DNS records first to preserve correct order
    $result=&$ds->ds->Execute("DELETE FROM zonedns
            WHERE id=$zoneid");

    // add reverse DNS into zonedns table
    for ($i = 1; $i < 11; $i++) {
        if ($hname[$i]) {
            $hnametemp=$hname[$i];

            // add DNS records
            $result=&$ds->ds->Execute("INSERT INTO zonedns
                    (id, hname, horder)
                    VALUES
                    ($zoneid,
                     ".$ds->ds->qstr($hnametemp).",
                     $i)");
        }
    }

    if ($result) {
        $ds->AuditLog(sprintf(my_("User %s modified zone %s."),
                    $HTTP_SERVER_VARS[AUTH_VAR], $zoneip));

        $ds->DbfTransactionEnd();
        insert($w,textbr(my_("Zone Modified")));
        $zone="";
    }
    else {
        $ds->DbfTransactionRollback();
        $formerror .= my_("Zone could not be modifed.  Try again.")."<br>";
    }
}
// ##################### END OF Edit ##############################

// ##################### Start OF Export Zone  ##############################
if ($action=="export") {
    // check if user belongs to customer admin group
    $result=$ds->GetCustomerGrp($cust);
    // can only be one row - does not matter if nothing is 
    // found as array search will return false
    $row = $result->FetchRow();
    if (!in_array($row["admingrp"], $grps)) {
        myError($w, my_("You may not add a zone as you are not a member of the customers admin group"));
    } 
    // Update serialcount
    $now = getdate();
    $newserialdate = $now[year] . str_pad($now[mon], 2, '0', STR_PAD_LEFT) . str_pad( $now[mday], 2, '0', STR_PAD_LEFT);
    if ($serialdate==$newserialdate) {
        $serialnum++;
    }else{
        $serialdate=$newserialdate;
        $serialnum=0;
    }

    // Update DNS Database Serial Count.  Update Serial Count only when we export.
    $result = &$ds->ds->Execute("UPDATE zones ".
            "set serialdate=".$ds->ds->qstr($serialdate).
            ", serialnum=$serialnum ".
            " WHERE customer=$cust AND id=$zoneid");


    $result = &$ds->ds->Execute("SELECT * FROM zones
            WHERE customer=$cust AND id=$zoneid");
    $row = $result->FetchRow();

    $tmpfname = tempnam (DNSEXPORTPATH, "revzone_") or 
        myError($w, my_("Could not create temporary file!"));
    $fp = fopen ("$tmpfname", "w");

    // header of document
    $output='<?xml version="1.0" ?>';
    fputs($fp, $output);
    fputs($fp, "\n");
    $ip=$row["zone"];
    list($octet1,$octet2,$octet3,$octet4) = explode(".",$ip);
    fputs($fp, sprintf('<zone domain="%s" zonesize="%s" octect1="%s" octect2="%s" octect3="%s" octect4="%s">', 
                $ip, $row["zonesize"], $octet1, $octet2, $octet3, $octet4));
    fputs($fp, "\n");

    // SOA portion
    fputs($fp, sprintf('<soa serialdate="%s" serialnum="%02d" ttl="%s" retry="%s" refresh="%s" expire="%s" minimumttl="%s" email="%s" />', 
                $serialdate, $serialnum, 
                $row["ttl"],
                $row["retry"],
                $row["refresh"],
                $row["expire"],
                $row["minimum"],
                $row["responsiblemail"]
                ));
    fputs($fp, "\n");

    // nameservers
    $result1 = &$ds->ds->Execute("SELECT hname FROM zonedns
            WHERE id=$zoneid
            ORDER BY horder");

    $cnt=0;
    while($row1 = $result1->FetchRow()) {
        fputs($fp, '<record type="NS">');
        fputs($fp, sprintf('<iphostname>%s</iphostname>', $row1["hname"]));
        fputs($fp, '</record>');
        fputs($fp, "\n");

        $cnt++;
    }
    if ($cnt < 2) {
        insert($w,textbr(my_("Invalid zone - zone should have at least two name servers defined")));

    }

    // get records from main ipplan ipaddr tables
    $result1 = &$ds->ds->Execute("SELECT ipaddr.ipaddr, ipaddr.hname
            FROM base, ipaddr
            WHERE base.customer = $cust AND
            base.baseindex = ipaddr.baseindex AND
            ipaddr.ipaddr >= ".$row["zoneip"]." AND
            ipaddr.ipaddr <= ".($row["zoneip"]+$row["zonesize"])."
            ORDER BY ipaddr.ipaddr");

    while($row1 = $result1->FetchRow()) {
        $ip=inet_ntoa($row1["ipaddr"]);

        // ignore blank records
        if (empty($row1["hname"])) {
            continue;
        }
        // test for valid domain name
        if (!preg_match('/^(([\w][\w\-\.]*)\.)?([\w][\w\-]+)(\.([\w][\w\.]*))?$/', $row1["hname"])) {

            insert($w,textbr(sprintf(my_("Invalid record - ignored: %s %s"), $ip, $row1["hname"])));
            continue;
        }
        fputs($fp, '<record type="PTR">');
        fputs($fp, sprintf('<host>%s</host>', $row1["hname"]));
        list($octet1,$octet2,$octet3,$octet4) = explode(".",$ip);
        fputs($fp, sprintf('<octet1>%s</octet1>', $octet1));
        fputs($fp, sprintf('<octet2>%s</octet2>', $octet2));
        fputs($fp, sprintf('<octet3>%s</octet3>', $octet3));
        fputs($fp, sprintf('<octet4>%s</octet4>', $octet4));
        fputs($fp, "\n");
        fputs($fp, sprintf('<iphostname>%s</iphostname>', $ip));
        fputs($fp, '</record>');
        fputs($fp, "\n");
    }

    // close zone
    fputs($fp, '</zone>');
    fputs($fp, "\n");

    fclose($fp);

    insert($w,textbr(sprintf(my_("Sent update to Backend Processor as file $s"), $tmpfname)));
}
// ##################### END OF Export Zone  ##############################

// Now Setup Page...
// start form
insert($w, $f1 = form(array("name"=>"THISFORM",
                            "method"=>"post",
                            "action"=>"$PHP_SELF")));

// ugly kludge with global variable!
$displayall=FALSE;
$cust=myCustomerDropDown($ds, $f1, $cust, $grps) or myError($w, my_("No customers"));

// get info from base table
$result = &$ds->ds->Execute("SELECT * FROM zones 
                             WHERE customer=$cust 
                             ORDER BY zone");

$totcnt=0;
// fastforward till first record if not first block of data
while ($block and $totcnt < $block*MAXTABLESIZE and
       $row = $result->FetchRow()) {
    DisplayBlock($w, $row, $totcnt, "&cust=".$cust.
                        "&rangeindex=".$rangeindex."&ipaddr=".$ipaddr.
                        "&descrip=".$descrip);
    $totcnt++;
}
insert($w,block("<p>"));

// create a table
insert($w,$tbig = table(array("cols"=>"1", "bgcolor"=>"#00CCFF")));
insert($tbig,$c = cell());
insert($c,$t = table(array("cols"=>"10",
                           "cellspacing"=>"1",
                           "bgcolor"=>"#99CCFF")));
// draw heading
insert($t,$c = cell());
insert($c,textb(my_("Zone")));
insert($t,$c = cell());
insert($c,textb(my_("Primary NS")));
insert($t,$c = cell());
insert($c,textb(my_("Secondary NS")));
insert($t,$c = cell());
insert($c,textb(my_("SerialDate")));
insert($t,$c = cell());
insert($c,textb(my_("TTL")));
insert($t,$c = cell());
insert($c,textb(my_("Refresh")));
insert($t,$c = cell());
insert($c,textb(my_("Retry")));
insert($t,$c = cell());
insert($c,textb(my_("Expire")));
insert($t,$c = cell());
insert($c,textb(my_("Min. TTL")));
insert($t,$c = cell());
insert($c,textb(my_("Action")));


$cnt=0;
while($row = $result->FetchRow()) {
setdefault("cell",array("bgcolor"=>color_flip_flop()));

    insert($t,$c = cell());
    insert($c,text($row["zone"]));

    $result1 = &$ds->ds->Execute("SELECT hname FROM zonedns
                                  WHERE id=".$row["id"]."
                                  ORDER BY horder");

    insert($t,$c = cell());
    $row1 = $result1->FetchRow();
    insert($c,text($row1["hname"]));

    insert($t,$c = cell());
    $row1 = $result1->FetchRow();
    insert($c,text($row1["hname"]));

    insert($t,$c = cell());
    insert($c,text($row["serialdate"].str_pad($row["serialnum"], 2, '0', STR_PAD_LEFT)));

    insert($t,$c = cell());
    insert($c,text($row["ttl"]));

    insert($t,$c = cell());
    insert($c,text($row["refresh"]));

    insert($t,$c = cell());
    insert($c,text($row["retry"]));

    insert($t,$c = cell());
    insert($c,text($row["expire"]));

    insert($t,$c = cell());
    insert($c,text($row["minimum"]));

    insert($t,$c = cell());
    insert($c,block("<small>"));
    insert($c,anchor($PHP_SELF."?cust=$cust&zoneid=".$row["id"]."&action=delete", my_("Delete Zone"),
                     $ipplanParanoid ? array("onclick"=>"return confirm('".my_("Are you sure?")."')") : FALSE));
 insert($c,block(" | "));
    insert($c,anchor("modifyzoneform.php?cust=$cust&zoneid=".$row["id"].
	"&action=edit".
	"&zone=".$row["zone"].
	"&size=".$row["zonesize"].
	"&serialdate=".$row["serialdate"].
	"&serialnum=".$row["serialnum"].
	"&ttl=".$row["ttl"].
	"&retry=".$row["retry"].
	"&refresh=".$row["refresh"].
	"&expire=".$row["expire"].
	"&minimum=".$row["minimum"].
	"&zonepath=".$row["zonefilepath1"].
	"&seczonepath=".$row["zonefilepath2"], my_("Edit Zone")));
 insert($c,block(" | "));
    insert($c,anchor($PHP_SELF."?cust=$cust&zoneid=".$row["id"]."&action=export".
	"&zone=".$row["zone"].
	"&serialdate=".$row["serialdate"].
	"&serialnum=".$row["serialnum"],
	my_("Export Zone"),
                     $ipplanParanoid ? array("onclick"=>"return confirm('".my_("Are you sure?")."')") : FALSE));
    insert($c,block("</small>"));

   $cnt++;
   $totcnt++;
   if ($totcnt % MAXTABLESIZE == MAXTABLESIZE-1)
      break;
}

insert($w,block("<p>"));

if (!$cnt) {
   insert($w,text(my_("Search found no Zone entries"), array("color"=>"red")));
}

insert($w, $f = form(array("method"=>"post",
                           "action"=>"modifyzoneform.php?cust=$cust&action=add")));
insert($f,submit(array("value"=>my_("Add a Zone"))));

$result->Close();
printhtml($w);

?>
